import pymol
from pymol import cmd 
from HIV_point import Point
import os
from calculate_trimer_coordinates import distance

def main(pngFilePath):
    print ('The default filename is points_2D.')
    fileName = input('Input file name:')
    inputFile = open(fileName,'r')
    inputFileList = inputFile.read().splitlines()
    pointMap = {}
    for i in inputFileList:
        tempList = i.split()
        pointMap[int(tempList[3])] = Point(float(tempList[0]),float(tempList[1]),float(tempList[2]),int(tempList[3]))
    Radius = float(inputFileList[0].split()[2])
    r = distance(pointMap[1],pointMap[2])

    print ('Eg: R.pdb or R.pse')
    outPutFile = input('Input output file name:')

    #Plot the first 4 trimer-center's coordinates.
    cmd.load('t1.pdb')
    #first trimerIndex
    cmd.select('/t1/PSDO')
    # cmd.color('blue','t1')
    cmd.show('line', 't1')
    cmd.rotate ('z', -30, 't1', camera=1)
    my_dict = {'L0':[]}
    cmd.iterate_state(1, '/t1/PSDO', 'L0.append( (x,y,z) )', space=my_dict)
    cmd.translate([0-my_dict['L0'][0][0],0-my_dict['L0'][0][1],0-my_dict['L0'][0][2]],'t1',camera=1)
    cmd.translate([0,0,Radius],'t1',camera=1)
    # cmd.translate([0,0,0],'t1',camera=1)

    #second trimerIndex
    cmd.create('t2', 't1')
    cmd.select('/t2/PSDO')
    cmd.show('line', 't2')
    cmd.translate([r, 0, 0], 't2', camera=1)
    cmd.orient('t2')
    cmd.rotate('z', 60, 't2', camera=1)
    cmd.rotate('z', -30 )

    # third trimerIndex
    cmd.create('t3','t2')
    cmd.select('/t3/PSDO')
    cmd.show('line','t3')
    cmd.origin('t1')
    cmd.rotate(pointMap[1].getCoordinate(),120,'t3',camera=1)

    # forth trimerIndex
    cmd.create('t4','t2')
    cmd.select('/t4/PSDO')
    cmd.show('line','t4')
    cmd.origin('t1')
    cmd.rotate(pointMap[1].getCoordinate(),240,'t4',camera=1)

    pointMap[1].setFather(0)
    pointMap[2].setFather(1)
    pointMap[3].setFather(1)
    pointMap[4].setFather(1)

    # According to the first 4 trimer-center coordinates, to plot the following trimers.
    keyList = pointMap.keys()
    keyList = sorted(keyList)
    for i in keyList:
        if (i>4):
            if (i%2)==1:
                tempFather = (i-1)/2
            else:
                tempFather = (i-2)/2
            tempTName = 't'+str(int(i))
            cmd.create(tempTName,'t1')
            if pointMap[tempFather].getFather()==1:
                pointMap[i].setFather(0)
            else:
                pointMap[i].setFather(1)
                cmd.rotate('z', 60, tempTName, camera=1)
                # cmd.rotate('z', -30 )
            cmd.select('/'+tempTName+'/PSDO')
            cmd.show('line',tempTName)
            cmd.translate([pointMap[i].x,pointMap[i].y,0],tempTName,camera=1)


    cmd.save(outPutFile)
    cmd.png(pngFilePath, ray = 1)
    
if __name__ == '__main__':
    main('/tmp/flattenHIV.png')